SUBROUTINE RLXSM(KIDIA,KFDIA,KLON,KLEVS,PWSAL,PWSATL,PWSASL,PWSARL,PWSATRL,PWSASRL,&
 & PWSAIL,PWSATIL,PWSASIL,ZRLXL,RXLSM)

!**** *RLXSM*

!     ---------------------------------------------------------------------

!     PURPOSE.
!     --------

!     Relaxation of soil moisture 

!     METHOD.
!     -------

!     Newtonian relaxation, following a paper by 
!     Krishnamurti et al., Tellus (1991), 43AB, 53-81
!
!     STEERING THE NUDGING OF SOIL MOISTURE
!
! LRXSM: PARAMETER INDICATING THE LEVELS TO BE NUDGED 
!
! LRXSM =  0: NO NUDGING (SUBROUTINE IS NOT ENTERED)      
! LRXSM = 12: 4XDAILY DATA AND 3 LEVELS (EXCLUDING LEVEL 1)      
! LRXSM = 13: 4XDAILY DATA AND 4 LEVELS
!
! LRXSMT: PARAMETER INDICATING THE TIME SCALE OF THE RELAXATION (IN HOURS)
!  
! LRXSMT =   0: ACTUAL VALUES RELPACED BY EXTERNAL ONES 
! LRXSMT =  24: 1 DAY
! LRXSMT = 120: 5 DAYS
!
! THE VALUES ARE SPECIFIED FOR EACH LEVEL SEPARATELY.
!
! LRXSMS: PARAMETER INDICATING WHEN THE RELAXATION IS DONE 
!
! LRXSMS = 0: RELAXATION BEFORE THE TIME STEP
! LRXSMS = 1: RELAXATION AFTER THE TIME STEP
!

!     EXTERNALS.
!     ----------

!     AUTHORS.
!     --------

!     W. MAY (CEC, LU) OCTOBER 2017 (based on I. RINGGAARD's subroutine RLXSI)

!     ---------------------------------------------------------------------

USE PARKIND1  , ONLY : JPIM, JPIB, JPRB
USE YOMHOOK   , ONLY : LHOOK,   DR_HOOK
USE YOMLUN    , ONLY : NULOUT   ,NULCO    ,NULERR
USE YOMMP     , ONLY : MYPROC
USE YOMDYN    , ONLY : TSTEP
USE YOMCT3    , ONLY : NSTEP

USE YOMRLXSM  , ONLY : LRXSM, LRXSMT1, LRXSMT2, LRXSMT3, LRXSMT4, LRXSMS
USE MPL_MODULE, ONLY : MPL_BROADCAST

USE YOMECEARTH, ONLY : LECEARTH,LECEARTH_UNCOUP
USE CPLNG

IMPLICIT NONE

INTEGER(KIND=JPIM), INTENT(IN)  :: KIDIA
INTEGER(KIND=JPIM), INTENT(IN)  :: KFDIA
INTEGER(KIND=JPIM), INTENT(IN)  :: KLON
INTEGER(KIND=JPIM), INTENT(IN)  :: KLEVS

REAL(KIND=JPRB), INTENT(IN) :: RXLSM(KLON) 

REAL(KIND=JPRB), INTENT(INOUT) :: ZRLXL(KLON,KLEVS) 
REAL(KIND=JPRB), INTENT(INOUT) :: PWSAL(KLON,KLEVS), PWSATL(KLON,KLEVS)

REAL(KIND=JPRB), INTENT(INOUT) :: PWSASL(KLON,KLEVS)
REAL(KIND=JPRB), INTENT(INOUT) :: PWSAIL(KLON,KLEVS), PWSATIL(KLON,KLEVS), &
 & PWSASIL(KLON,KLEVS)
REAL(KIND=JPRB), INTENT(INOUT) :: PWSARL(KLON,KLEVS), PWSATRL(KLON,KLEVS), &
 & PWSASRL(KLON,KLEVS)

! Local variables 

INTEGER(KIND=JPIM) :: JL, JLS, LRXSMTX

REAL (KIND=JPRB) :: PWSAX, PWSAY, RTIMH1, RTIMH2, RTIMH3, RTIMH4, RTIMHX 
REAL (KIND=JPRB) :: RTIMRLX, TWTSTEP, ZFACT

LOGICAL :: LLFIRST

REAL(KIND=JPRB) :: ZHOOK_HANDLE

#include "fcttim.h"

IF (LHOOK) CALL DR_HOOK('RLXSM',0,ZHOOK_HANDLE)

IF (NSTEP == 0) THEN
   LLFIRST = .TRUE.
ELSE
   LLFIRST = .FALSE.
ENDIF

IF (LLFIRST) THEN
   TWTSTEP = TSTEP
ELSE
   TWTSTEP = 2.*TSTEP
ENDIF

RTIMH1 = FLOAT(LRXSMT1)
RTIMH2 = FLOAT(LRXSMT2)
RTIMH3 = FLOAT(LRXSMT3)
RTIMH4 = FLOAT(LRXSMT4)

JLS = 1

IF ( LRXSM == 12 ) THEN

   DO JL=KIDIA,KFDIA
      IF (RXLSM(JL) > 0.5_JPRB) THEN

         PWSASL(JL,JLS) = PWSAL(JL,JLS)+TSTEP*PWSATL(JL,JLS)

         PWSARL(JL,JLS) = PWSAL(JL,JLS) 
         PWSATRL(JL,JLS) = PWSATL(JL,JLS)
         PWSASRL(JL,JLS) = PWSASL(JL,JLS)

         PWSAIL(JL,JLS) = 0.0_JPRB
         PWSATIL(JL,JLS) = 0.0_JPRB
         PWSASIL(JL,JLS) = 0.0_JPRB

      ENDIF
   ENDDO 

ELSE
 
   IF (LRXSMT1 == 0) THEN

      DO JL=KIDIA,KFDIA

         IF (RXLSM(JL) > 0.5_JPRB) THEN

            PWSASL(JL,JLS) = PWSAL(JL,JLS)+TSTEP*PWSATL(JL,JLS)
            PWSASRL(JL,JLS) = ZRLXL(JL,JLS)
            PWSASIL(JL,JLS) = PWSASL(JL,JLS)-PWSASRL(JL,JLS) 
            PWSAX = PWSASRL(JL,JLS)-TSTEP*PWSATL(JL,JLS)

            IF (PWSAX >= 0.0_JPRB ) THEN
               PWSARL(JL,JLS) = PWSAX
               PWSATRL(JL,JLS) = PWSATL(JL,JLS)
               PWSAIL(JL,JLS) = PWSAL(JL,JLS)-PWSARL(JL,JLS)
               PWSATIL(JL,JLS) = 0.0_JPRB
            ELSE 
               PWSARL(JL,JLS) = 0.0_JPRB
               PWSAY = PWSASRL(JL,JLS)/TSTEP 
               PWSATRL(JL,JLS) = PWSAY
               PWSAIL(JL,JLS) = PWSAL(JL,JLS)-PWSARL(JL,JLS)
               PWSATIL(JL,JLS) = PWSATL(JL,JLS)-PWSATRL(JL,JLS)
            ENDIF

         ENDIF
      ENDDO
   ELSE

      RTIMRLX = 1./(3600.*RTIMH1) 
      ZFACT = 1./(1.+TWTSTEP*RTIMRLX)

      DO JL=KIDIA,KFDIA
         IF (RXLSM(JL) > 0.5_JPRB) THEN
 
            PWSASL(JL,JLS) = PWSAL(JL,JLS)+TSTEP*PWSATL(JL,JLS)
            PWSASRL(JL,JLS) = &
                 & (PWSASL(JL,JLS)+TWTSTEP*RTIMRLX*ZRLXL(JL,JLS))*ZFACT
            PWSASIL(JL,JLS) = PWSASL(JL,JLS)-PWSASRL(JL,JLS) 
            PWSAX = PWSASRL(JL,JLS)-TSTEP*PWSATL(JL,JLS)

            IF (PWSAX >= 0.0_JPRB ) THEN
               PWSARL(JL,JLS) = PWSAX
               PWSATRL(JL,JLS) = PWSATL(JL,JLS)
               PWSAIL(JL,JLS) = PWSAL(JL,JLS)-PWSARL(JL,JLS)
               PWSATIL(JL,JLS) = 0.0_JPRB
            ELSE 
               PWSARL(JL,JLS) = 0.0_JPRB
               PWSAY = PWSASRL(JL,JLS)/TSTEP 
               PWSATRL(JL,JLS) = PWSAY
               PWSAIL(JL,JLS) = PWSAL(JL,JLS)-PWSARL(JL,JLS)
               PWSATIL(JL,JLS) = PWSATL(JL,JLS)-PWSATRL(JL,JLS)
            ENDIF
         ENDIF
      ENDDO

   ENDIF
ENDIF

!*
!     Other levels
!     ------------

DO JLS=2,KLEVS
 
   IF (JLS == 2) THEN
      LRXSMTX = LRXSMT2
      RTIMHX = RTIMH2
   ELSEIF (JLS == 3) THEN
      LRXSMTX = LRXSMT3
      RTIMHX = RTIMH3
   ELSEIF (JLS == 4) THEN
      LRXSMTX = LRXSMT4
      RTIMHX = RTIMH4
   ENDIF

   IF (LRXSMTX == 0) THEN

      DO JL=KIDIA,KFDIA

         IF (RXLSM(JL) > 0.5_JPRB) THEN
   
            PWSASL(JL,JLS) = PWSAL(JL,JLS)+TSTEP*PWSATL(JL,JLS)
            PWSASRL(JL,JLS) = ZRLXL(JL,JLS)
            PWSASIL(JL,JLS) = PWSASL(JL,JLS)-PWSASRL(JL,JLS) 
            PWSAX = PWSASRL(JL,JLS)-TSTEP*PWSATL(JL,JLS)

            IF (PWSAX >= 0.0_JPRB ) THEN
               PWSARL(JL,JLS) = PWSAX
               PWSATRL(JL,JLS) = PWSATL(JL,JLS)
               PWSAIL(JL,JLS) = PWSAL(JL,JLS)-PWSARL(JL,JLS)
               PWSATIL(JL,JLS) = 0.0_JPRB
            ELSE 
               PWSARL(JL,JLS) = 0.0_JPRB
               PWSAY = PWSASRL(JL,JLS)/TSTEP 
               PWSATRL(JL,JLS) = PWSAY
               PWSAIL(JL,JLS) = PWSAL(JL,JLS)-PWSARL(JL,JLS)
               PWSATIL(JL,JLS) = PWSATL(JL,JLS)-PWSATRL(JL,JLS)
            ENDIF
         ENDIF
      ENDDO

   ELSE

      RTIMRLX = 1./(3600.*RTIMHX) 
      ZFACT = 1./(1.+TWTSTEP*RTIMRLX)

      DO JL=KIDIA,KFDIA

         IF (RXLSM(JL) > 0.5_JPRB) THEN
 
            PWSASL(JL,JLS) = PWSAL(JL,JLS)+TSTEP*PWSATL(JL,JLS)
            PWSASRL(JL,JLS) = &
                 & (PWSASL(JL,JLS)+TWTSTEP*RTIMRLX*ZRLXL(JL,JLS))*ZFACT
            PWSASIL(JL,JLS) = PWSASL(JL,JLS)-PWSASRL(JL,JLS) 
            PWSAX = PWSASRL(JL,JLS)-TSTEP*PWSATL(JL,JLS)

            IF (PWSAX >= 0.0_JPRB ) THEN
               PWSARL(JL,JLS) = PWSAX
               PWSATRL(JL,JLS) = PWSATL(JL,JLS)
               PWSAIL(JL,JLS) = PWSAL(JL,JLS)-PWSARL(JL,JLS)
               PWSATIL(JL,JLS) = 0.0_JPRB
            ELSE 
               PWSARL(JL,JLS) = 0.0_JPRB
               PWSAY = PWSASRL(JL,JLS)/TSTEP 
               PWSATRL(JL,JLS) = PWSAY
               PWSAIL(JL,JLS) = PWSAL(JL,JLS)-PWSARL(JL,JLS)
               PWSATIL(JL,JLS) = PWSATL(JL,JLS)-PWSATRL(JL,JLS)
            ENDIF
         ENDIF
      ENDDO
      
   ENDIF
ENDDO

WRITE(NULOUT,*) 'RLXSM finished'

CALL FLUSH(NULOUT)

IF (LHOOK) CALL DR_HOOK('RLXSM',1,ZHOOK_HANDLE)
END SUBROUTINE RLXSM
