#!/bin/ksh
#
#   Job to retrieve archived near-surface meteorological forcing
#

set -eux 

gridTF(){

NLAT=$(grib_ls -x -p N -w count=1 $1 | head -n3 | tail -n1 | awk '{print $1}' )
isO=$(grib_ls -x -p isOctahedral -w count=1 $1 | head -n3 | tail -n1 | awk '{print $1}')
GT=N
if [[ $isO == 1 ]] ; then GT=O ; fi
echo ${GT}${NLAT}
}

# to avoid problems after "module load python"... 
alias datediff=/usr/local/bin/datediff


##===================================
## HARDCODED variables 
##===================================
# EPS="0.000000000001"               # small number 
EPS="0.000001"                     # small number for precip in kg.m-2.s-1 == 0.086 mm/day 
NFPDAY=2                           # NFPDAY : # forecast per day 
##=====================================

##===================================
## Variables: can be changes vi environment in calling script 
##===================================
FORCINGLEV=${FORCINGLEV:-0}      # model level to extract data: 0: lowest model level, 1: 2nd lowest model level..
GRID=${GRID:-XXX}                 # grid  
LEXTRACT_ML=${LEXTRACT_ML:-true}  # extract model level: t/u/v/q 
LEXTRACT_LNSP=${LEXTRACT_LNSP:-true}  # extract model level: t/u/v/q 
LEXTRACT_SFC=${LEXTRACT_SFC:-true} # extract surface : SSRD/STRD/LSP/CP/SF/SP
BINS=${BINS:-'.'}                 # location of binaries / scripts / 


## OD: 91 levels until 24 June 2013 
#     137 levels for 25 June 2013 expver 62 - esuite
#     137 levels for 26 June 2013 onwards 

##===================================
## Default values - can be changes via command line input
##==================================
EXTDATE="20100101" # extraction date: YYYYMMDD for daily / YYYMMDD00 for monthly chunk
EXPVER='1'         # experiment version
CLASS=xxx          # class: ERA5 only 
FREQ=3             # hourly frequency only  
DATADIR='.'        # directory to save output 
STREAM='oper'      # forecast stream to extract  
LEXTRACT_LAP=false # extract and compute lapse rate 
ENSMEM=-1          # ensemble members

##===================================
## Change default via command line options
##===================================


while getopts e:i:d:f:c:s:l:n: option
do
  case $option in
    e) EXTDATE=$OPTARG;;
    i) EXPVER=$OPTARG;;
    d) DATADIR=$OPTARG;;
    f) FREQ=$OPTARG;;
    c) CLASS=$OPTARG;;
    s) STREAM=$OPTARG;;
    l) LEXTRACT_LAP=$OPTARG;;
    n) ENSMEM=$OPTARG;;
    \?) exit 1;;
  esac
done

echo '============================='
echo '        OPTIONS              '
echo "-e EXTDATE=$EXTDATE" 
echo "-i EXPVER=$EXPVER" 
echo "-d DATADIR=$DATADIR "
echo "-f FREQ=$FREQ"
echo "-c CLASS=$CLASS"
echo "-s STREAM=$STREAM"
echo "-l LEXTRACT_LAP=$LEXTRACT_LAP"
echo "-n ENSMEM=$ENSMEM"
echo ''


## special fix for od in change of vertical resolution
# if [[ $EXPVER = 1 && $CLASS = od && $STREAM = oper && $EXTDATE = 20130625 ]]; then
#   EXPVER=62
# fi 
## special case of EI close to real-time 
#if [[ $EXPVER = 1 && $CLASS = ei && $STREAM = oper && $EXTDATE -gt 20160800 ]]; then
#  EXPVER=2426
#fi


##=======================================
## Resolution related 
##========================================

if [[ $GRID == XXX ]]; then
  MGRID=""
else
  MGRID="grid=${GRID},"
fi

##=======================================
## PATHs
##========================================

mkdir -p $DATADIR

##=======================================
## Ensemble enabling
ensmem=""
[[ $STREAM = @(ENDA|enda|ELDA|elda) ]] && ensmem="number=$ENSMEM,"


##====================================
## define date/time variables 
##===================================
YYYY=$( echo ${EXTDATE} | cut -c1-4)
MM=$( echo ${EXTDATE} | cut -c5-6)
DD=$(echo ${EXTDATE} | cut -c7-8)

if [[ $DD = "00" ]]; then # Monthly extraction 
  D1=01
  D2=$( dateincr $( dateincr ${YYYY}${MM}${D1} +33 | cut -c1-6 )01 -1  | cut -c 7-8)
else # daily extraction 
  D1=$DD
  D2=$DD
fi

if [[ $NFPDAY == 2 ]]; then
  
  DATE1=${YYYY}${MM}${D1}  # 1st date
  DATE2=${YYYY}${MM}${D2}  # las date 
  if [[ $CLASS = ea ]]; then
    if [[ $FREQ != @(1|3) ]]; then
        echo "Attention, for ERA5 forcing only hourly or 3-hourly analysis are available!"
        echo "Check your FREQ variable in prepIFS"
        exit -1
     fi

    TIME="06/18"  # time to extract 
    FORMULA_SSRD="max(0,(SSRD2))/(${FREQ}*3600)"
    FORMULA_STRD="(STRD2)/(${FREQ}*3600)"
    FORMULA_LSP="1000.*max(0,(LSP2))/(${FREQ}*3600)"
    FORMULA_CP="1000.*max(0,(CP2))/(${FREQ}*3600)"
    FORMULA_SF="1000.*max(0,(SF2))/(${FREQ}*3600)"
    
    STEPS_VAR=$( echo $(seq -w $FREQ  $FREQ 12 ) | sed "s/ /\//g")  # steps to retrieve ML
    STEPS_FLX=$STEPS_VAR  # steps to retrieve SFC
    STEPS_FLXn=$STEPS_FLX  # steps to retrieve SFC   
    STEPS_FLXp=$STEPS_FLX  # steps to retrieve SFC   

    ## special case for 1st 6 hours, we need the previous day forecast     
    STEPS_VAR0=$( echo $(seq -w 6 $FREQ 12 ) | sed "s/ /\//g")  # ML
    STEPS_FLX0=$STEPS_VAR0 # SFC
    STEPS_FLX0n=$STEPS_VAR0
    STEPS_FLX0p=$STEPS_VAR0
    DATE0=$( dateincr ${DATE1} -1)
    TIME0="18"
    CLASS0=$CLASS
    EXPVER0=$EXPVER
    if [[ $DATE1 == 19790101 && $EXPVER == 1 ]]; then
        EXPVER0=2931
    fi
    #if [[ $DATE1 == 20000101 && $EXPVER == 1 ]]; then
    #    EXPVER0=2504
    #fi
  
  else # not ea 
  
    TIME="00/12"  # time to extract 
    FORMULA_SSRD="max(0,(SSRD2-SSRD1))/(${FREQ}*3600)"
    FORMULA_STRD="(STRD2-STRD1)/(${FREQ}*3600)"
    FORMULA_LSP="1000.*max(0,(LSP2-LSP1))/(${FREQ}*3600)"
    FORMULA_CP="1000.*max(0,(CP2-CP1))/(${FREQ}*3600)"
    FORMULA_SF="1000.*max(0,(SF2-SF1))/(${FREQ}*3600)"
    ## special case for ei 
    if [[ $CLASS == ei ]]; then
      STEPS_VAR=$( echo $(seq -w 3  $FREQ 12 ) | sed "s/ /\//g")  # steps to retrieve ML
      STEPS_FLX=$( echo $(seq -w 9  $FREQ 21 ) | sed "s/ /\//g")  # steps to retrieve SFC
      STEPS_FLXn=$( echo $(seq -w 12 $FREQ 21  ) | sed "s/ /\//g")  # steps to retrieve SFC   
      STEPS_FLXp=$( echo $(seq -w 9 $FREQ 18 ) | sed "s/ /\//g")  # steps to retrieve SFC   

      ## special case for 1st 6 hours, we need the previous day forecast     
      STEPS_VAR0=$( echo $(seq -w 12 $FREQ 12 ) | sed "s/ /\//g")  # ML
      STEPS_FLX0=$( echo $(seq -w 9  $FREQ 21 ) | sed "s/ /\//g")  # steps to retrieve SFC
      STEPS_FLX0n=$( echo $(seq -w 12 $FREQ 21  ) | sed "s/ /\//g")  # steps to retrieve SFC   
      STEPS_FLX0p=$( echo $(seq -w 9 $FREQ 18 ) | sed "s/ /\//g")  # steps to retrieve SFC   
      DATE0=$( dateincr ${DATE1} -1)
      TIME0="12"
      CLASS0=$CLASS
      EXPVER0=$EXPVER
      if [[ $DATE0 -lt 19790101 ]]; then
        CLASS0='e4'
        GRID=N128
        MGRID="grid=${GRID},"
      fi
    
    else
      STEPS_VAR=$( echo $(seq -w $FREQ  $FREQ 12 ) | sed "s/ /\//g")  # steps to retrieve ML
      STEPS_FLX=$( echo $(seq -w 0  $FREQ 12 ) | sed "s/ /\//g")  # steps to retrieve SFC
      STEPS_FLXn=$( echo $(seq -w $FREQ $FREQ 12 ) | sed "s/ /\//g")  # steps to retrieve SFC   
      STEPS_FLXp=$( echo $(seq -w 0 $FREQ 11 ) | sed "s/ /\//g")  # steps to retrieve SFC   

      ## special case for 1st 6 hours, we need the previous day forecast     
      STEPS_VAR0=$( echo $(seq -w 12 $FREQ 12 ) | sed "s/ /\//g")  # ML
      STEPS_FLX0=$( echo "$(( 12 -$FREQ)) 12" | sed "s/ /\//g") # SFC
      STEPS_FLX0n=$( echo "12" | sed "s/ /\//g") # SFC
      STEPS_FLX0p=$( echo "$(( 12 -$FREQ))" | sed "s/ /\//g") # SFC
      DATE0=$( dateincr ${DATE1} -1)
      TIME0="12"
      CLASS0=$CLASS
      EXPVER0=$EXPVER
    fi # end ei special case 
  fi # end of ea special case 
fi


mars << EOF
list, 
stream=$STREAM,
class=$CLASS,
expver=$EXPVER,
date=$DATE1,
time=0, 
type=an,
step=0,
$ensmem
levtype=ml,
levelist=1/to/137,
param=t,
target="list"
EOF
FORCINGLEV0=$( cat list | grep "Entries" | awk '{print $3}')
FORCINGLEV=$(( $FORCINGLEV0 - $FORCINGLEV )) 
rm -f list 
   
case $FORCINGLEV0 in
   60) DTDZLEV=47   ;; # 
   62) DTDZLEV=46   ;; # 
   91) DTDZLEV=75   ;; # 
  137) DTDZLEV=111 ;; # 
esac


FSAVE=""  # list of variables to save 

##====================================
## Extract SFC
##====================================
if [[ $LEXTRACT_SFC = true ]]; then
echo "Extracting SFC" 
rm -f bar
touch bar 
if [[ $DATE0 != NONE ]]; then  ## special data0
cat > bar << EOF
retrieve,
stream=$STREAM,
class=$CLASS0,
expver=$EXPVER0,
date=$DATE0,
time=$TIME0,
step=$STEPS_FLX0,
type=fc,
$MGRID
$ensmem
levtype=sfc,
param=SSRD/STRD/CP/LSP/SF,
target="tmp0.grb"
EOF
fi
cat >> bar << EOF
retrieve, 
stream=$STREAM,
class=$CLASS,
expver=$EXPVER,
date=$DATE1/TO/$DATE2,
time=${TIME}, 
step=$STEPS_FLX,
type=fc,
levtype=sfc,
$MGRID
$ensmem
param=SSRD/STRD/CP/LSP/SF,
target="tmp1.grb"
EOF
mars < bar 

rm -f bar
touch bar

for pp in STRD SSRD LSP CP SF
do
eval "formula=\$FORMULA_$pp" 
cat >> bar << EOF
read,source="tmp0.grb",param=$pp,step=$STEPS_FLX0p,fieldset=${pp}1
read,source="tmp0.grb",param=$pp,step=$STEPS_FLX0n,fieldset=${pp}2
compute,fieldset=${pp}f,formula="$formula",accuracy=16
EOF
done
cat >> bar << EOF
write,fieldset=STRDf,target="LWdown.grb"
write,fieldset=SSRDf,target="SWdown.grb"
write,fieldset=SFf,target="Snowf.grb"
compute,formula="max(0,CPf+LSPf-SFf)",fieldset=TP,accuracy=16
write,fieldset=TP,target="Rainf.grb"
compute,formula="max(0,min(1,(CPf>$EPS)*((CPf+$EPS)/($EPS+CPf+LSPf))+0*(CPf<=$EPS)))",fieldset=Ctpf,accuracy=16
write,fieldset=Ctpf,target="Ctpf.grb"
EOF
for pp in STRD SSRD LSP CP SF
do
eval "formula=\$FORMULA_$pp" 
cat >> bar << EOF
read,source="tmp1.grb",param=$pp,step=$STEPS_FLXp,fieldset=${pp}1
read,source="tmp1.grb",param=$pp,step=$STEPS_FLXn,fieldset=${pp}2
compute,fieldset=${pp}f,formula="$formula",accuracy=16
EOF
done
cat >> bar << EOF
write,fieldset=STRDf,target="LWdown.grb"
write,fieldset=SSRDf,target="SWdown.grb"
write,fieldset=SFf,target="Snowf.grb"
compute,formula="max(0,CPf+LSPf-SFf)",fieldset=TP,accuracy=16
write,fieldset=TP,target="Rainf.grb"
compute,formula="max(0,min(1,(CPf>$EPS)*((CPf+$EPS)/($EPS+CPf+LSPf))+0*(CPf<=$EPS)))",fieldset=Ctpf,accuracy=16
write,fieldset=Ctpf,target="Ctpf.grb"
EOF
mars < bar  
rm -f tmp0.grb tmp1.grb 

# clean 
rm -f forcing_sfc_aux.grb bar
echo 
FSAVE="${FSAVE} Rainf Snowf LWdown SWdown Ctpf"
fi # LEXTRACT_SFC




##====================================
## Extract model level data 
##===================================
if [[ $LEXTRACT_ML = true ]] ; then

if [[ $GRID == XXX ]]; then
  if [[ $LEXTRACT_SFC = true ]] ; then
    MGRID="grid=$( gridTF LWdown.grb ),"
  else
    mars << EOF
    retrieve,stream=$STREAM,class=$CLASS0,expver=$EXPVER0,
    date=$DATE0,time=0,step=0,type=an,levtype=sfc,param=lsm,$ensmem
    target="tmp_lsm.grb"
EOF
    MGRID="grid=$( gridTF tmp_lsm.grb ),"
  fi 
else
  MGRID="grid=${GRID},"
fi

echo "Extracting ML"
rm -f bar 
touch bar
if [[ $DATE0 != NONE ]]; then  ## special data0
cat >> bar << EOF
retrieve, 
stream=$STREAM,
class=$CLASS0,
expver=$EXPVER0,
date=$DATE0,
time=${TIME0}, 
step=$STEPS_VAR0,
type=fc,
levtype=ml,
levelist=$FORCINGLEV,
$MGRID
$ensmem
param=u/v/t/q,
target="forcing_ml_aux.grb"
EOF
if [[ $LEXTRACT_LNSP = true ]]; then
cat >> bar << EOF 
retrieve,levelist=1,param=lnsp
EOF
fi 

fi
cat >> bar << EOF
retrieve, 
stream=$STREAM,
class=$CLASS,
expver=$EXPVER,
date=$DATE1/TO/$DATE2,
time=${TIME}, 
step=$STEPS_VAR,
type=fc,
levtype=ml,
levelist=$FORCINGLEV,
$MGRID
$ensmem
param=u/v/t/q,
target="forcing_ml_aux.grb"
EOF
if [[ $LEXTRACT_LNSP = true ]]; then
cat >> bar << EOF 
retrieve,levelist=1,param=lnsp
EOF
fi 
mars < bar
if [[ $LEXTRACT_LNSP = true ]]; then
grib_copy -w shortName=lnsp forcing_ml_aux.grb PSurf.grb
fi 
grib_copy -w shortName=t forcing_ml_aux.grb Tair.grb
grib_copy -w shortName=q forcing_ml_aux.grb  Qair.grb
grib_copy -w shortName=u forcing_ml_aux.grb  Wind_E.grb
grib_copy -w shortName=v forcing_ml_aux.grb  Wind_N.grb
# mars << EOF
# read,source="forcing_ml_aux.grb",param=u,fieldset=u
# read,source="forcing_ml_aux.grb",param=v,fieldset=v
# compute,accuracy=16,formula="sqrt(u*u+v*v)",fieldset=wind
# write,fieldset=wind,target="Wind.grb"
# EOF

FSAVE="${FSAVE} Wind_E Wind_N Tair Qair"
if [[ $LEXTRACT_LNSP = true ]]; then
    FSAVE="${FSAVE} PSurf"
fi

rm -f forcing_ml_aux.grb bar bar1 

fi # LEXTRACT_MET


cat >filter.rules << EOF
transient hours  = dataTime / 100 + stepRange ;

if ( (dataDate == $DATE0 && hours >=24 )        || 
     (dataDate >= $DATE1 && dataDate < $DATE2 ) || 
     (dataDate == $DATE2 && hours <=24 )          )
  { write; }
EOF

for ff in $FSAVE
do 
  grib_filter -o tmp.grb filter.rules ${ff}.grb
  mv tmp.grb ${ff}.grb
done
rm -f filter.rules

##====================================
## Extract Lapse rate 
##===================================
if [[ $LEXTRACT_LAP = true ]] ; then
echo "Extracting LAP"
rm -f bar 
touch bar

cat >> bar << EOF
retrieve, 
stream=$STREAM,
class=$CLASS,
expver=$EXPVER,
date=$DATE1/TO/$DATE2,
time=0/6/12/18, 
type=an,
levtype=ml,
levelist=${DTDZLEV}/to/${FORCINGLEV0},
$MGRID
$ensmem
param=t/q,
target="[param]_lev.grb"
retrive,
levelist=1,
param=lnsp,
target="[param]_lev.grb"
EOF
mars < bar 

rm -f bar
touch bar 

for pp in 130 133 152
do
for tt in 00 06 12 18
do
cat >> bar << EOF
read,source="${pp}_lev.grb",param=${pp},time=${tt},fieldset=f_${tt}_${pp}
EOF
done
cat >> bar << EOF
compute,formula="(f_00_${pp}+f_06_${pp}+f_12_${pp}+f_18_${pp})/4.",fieldset=${pp}F
write,fieldset=${pp}F,target="${pp}_levM.grb"
EOF
done 
mars < bar 

compute_lapse_rate.py -ftlev 130_levM.grb -fqlev 133_levM.grb -fplev 152_levM.grb -o .

FSAVE="${FSAVE} lapseS lapseM"
rm -f 133_* 130_* 152_* bar tmp.grb 
fi # LEXTRACT_LAP

###=================================
### save
for ff in $FSAVE
do
  mv ${ff}.grb $DATADIR/${ff}.grb 
done


###=================================
### Final clean
