#!/usr/bin/env python

# E. Dutra Dec 2018
# Script to limit specific humidity to 1-99% relative humidity 
# It used the ECMWF formulation to compute saturation vapor pressure 

# IMPORTANT: the input Qair file with specific humidity is updated no backup copy is made ! 



from netCDF4 import Dataset,date2num
import numpy as np


#CONSTANTS:
RLVTT=2.5008E+6
RG=9.80665            # Gravitational Acceleration [m/s2]
RKBOL=1.380658E-23    # Boltzmann constant (k) [J/K]
RNAVO=6.0221367E+23   # Avogadro constant (N_A) [1/mol]
R=RNAVO*RKBOL         # Perfect gas constant (= 8.314511211948600)
RMD=28.9644           # Dry air mass
RMV=18.0153           # Vapour  mass
RD=1000.0*R/RMD       # Dry air cst. (= 287.0596736665907 J/kg/K)
RV=1000.0*R/RMV       # Vapour  cst. (= 461.5249933083879 J/kg/K)
RETV=RV/RD-1.0        # Rv/Rd-1 (= 0.608)
RTT=273.16            # Temperature of water fusion at normal pressure
RTWAT=RTT             # Criteria for temperature with respect to liquid water
RTICE=RTT-23.0        # Criteria for temperature with respect to ice
R2ES=611.21           # Constant in Teten's formula: eq(7.5)
R3LES=17.5020         # Mixing Ratio Over Liquid Water in Teten's formula: eq(7.5)
R4LES=32.190         # Constant in Teten's formula for liquid water: eq(7.5)
R3IES=22.5870        # Mixing Ratio Over Ice in Teten's formula: eq(7.5)
R4IES=-0.70          # Constant in Teten's formula for ice: eq(7.5)
PRHMAX=0.999          # max relative humidity
#PRHMAX=2          # max relative humidity
PRHMIN=0.001          # Min relative humidiy

def calc_rh_t_td(t,td):
  return np.maximum(PRHMIN,np.minimum(PRHMAX,np.exp(-RLVTT/(RV*t*td)*(t-td))))

def calc_qair(rh,t,p):
  pes=calc_PES(t)
  return rh*pes / ( p*(RETV+1.)-rh*pes*RETV)

def calc_rh_p_q_t(p,q,t):
  pes = calc_PES(t)
  return np.maximum(PRHMIN,np.minimum(PRHMAX,(p*q*(RETV+1.)) / ((1.+RETV*q)*pes) ) )

def calc_PES(Tair):
  
  # ratio between water and ice : Alpha in eq(7.73) for IFS Document Cy38R1e staturation vapor
  FOEALFA = np.minimum(1.0,( (np.maximum(
                       RTICE,np.minimum(RTWAT,Tair))-RTICE) /(RTWAT-RTICE))**2.0)
  #pressure with Teten's formula, eq. 7.15 for IFS doc CY38R1
  FOEEWM = R2ES *(FOEALFA*np.exp(R3LES*(Tair-RTT)/(Tair-R4LES))+
           (1.0-FOEALFA)*np.exp(R3IES*(Tair-RTT)/(Tair-R4IES)))
  return FOEEWM

def read_args():
  import argparse
  parser = argparse.ArgumentParser(description='')
  parser.add_argument('-iQ',dest='fqin',default='NONE',type=str,metavar='fqin',
                      help='input/ouput Q : content of the file is updated')
  parser.add_argument('-iT',dest='ftin',default='NONE',type=str,metavar='ftin',
                      help='input quair file ')
  parser.add_argument('-iP',dest='fpin',default='NONE',type=str,metavar='fpin',
                      help='input lnsp file ')
  args =  parser.parse_args()
  return args

args = read_args()
fqin=args.fqin
ftin=args.ftin
fpin=args.fpin


nct = Dataset(ftin,'r')
ncp = Dataset(fpin,'r')
ncq = Dataset(fqin,'r+')

nt = len(nct.dimensions['time'])
for it in range(nt):
  tair = nct.variables['Tair'][it,:]
  qair = ncq.variables['Qair'][it,:]
  psurf = ncp.variables['PSurf'][it,:]
  qmax = calc_qair(tair*0.+0.999,tair,psurf)
  qnew=np.minimum(qair,qmax)
  ncq.variables['Qair'][it,:]=np.minimum(qair,qmax)
  #print it
nct.close()
ncp.close()
ncq.close()
